#include "Dictionary.h"

Dictionary::Dictionary() :_parameter(NULL), _value(NULL), _next(NULL), _head(this)
{

}

Dictionary::Dictionary(const char *parameter, const char *value) : Dictionary()
{
    //We copy  the parameter and the value
    _parameter = (char *) malloc((strlen(parameter) * sizeof(char)) + 1); //+1 for the string terminating character
    _value = (char *) malloc((strlen(value) * sizeof(char)) + 1);

    strcpy(_parameter, parameter);
    strcpy(_value, value);
}

Dictionary::Dictionary(Dictionary const& dictionaryToCopy) //Copy constructor needed because of pointers
{
    _head = NULL;
    _next = NULL;

    _parameter = (char *) malloc((strlen(dictionaryToCopy._parameter) * sizeof(char)) + 1); //+1 for the string terminating character
    _value = (char *) malloc((strlen(dictionaryToCopy._value) * sizeof(char)) + 1);

    strcpy(_parameter, dictionaryToCopy._parameter);
    strcpy(_value, dictionaryToCopy._value);
}

Dictionary::~Dictionary()
{
    if(_head == this)
      dispose();
    free(_parameter);
    //_parameter = NULL; Useless, just my c habits
    free(_value);
    //_value = NULL; Useless, just my c habits
}

boolean Dictionary::addParameter(const char *parameter, const char *value)
{
    Dictionary *dictionaryNode = new Dictionary(parameter, value);
    return addNewNodeAtTheEnd(dictionaryNode);
}

Dictionary Dictionary::get(const char *parameter)
{
    if(isListEmpty(_head->_next))return Dictionary();

    Dictionary *cursor = _head->_next;

    while(!isListEmpty(cursor))
    {
        if(strcmp(cursor->_parameter,parameter) == 0)
            return *cursor;
        cursor = cursor->_next;
    }

    return Dictionary();
}

Dictionary Dictionary::operator()(const char *parameter)
{
    return get(parameter);
}

Dictionary Dictionary::get(const unsigned int index)
{
    unsigned int position(0);
    if(isListEmpty(_head->_next))return Dictionary();

    Dictionary *cursor = _head->_next;

    while(!isListEmpty(cursor))
    {
        if(position++ == index)
            return *cursor;
        cursor = cursor->_next;
    }

    return Dictionary();
}

Dictionary Dictionary::operator()(const unsigned int index)
{
    return get(index);
}

unsigned int Dictionary::count()
{
    unsigned int counter(0);
    if(isListEmpty(_head->_next))return counter;

    Dictionary *cursor = _head->_next;
    while(!isListEmpty(cursor))
    {
        counter++;
        cursor = cursor->_next;
    }
    return counter;
}

boolean Dictionary::addNewNodeAtTheEnd(Dictionary *node)
{
    if(node == NULL) return false;

    node->_head = _head; //Every node should point to the first node

    if(_next == NULL) //This is our first node then
    {
        _next = node;
        return true;
    }

    // /!\ We have to work with the _next reference in the loop, if we don't it won't work as expected
    Dictionary *cursor = _head;
    while(!isListEmpty(cursor->_next))
    {
        if(strcmp(cursor->_next->_parameter, node->_parameter) == 0)//If we find the same parameter name, we replace it
        {
            free(cursor->_next->_value);
            cursor->_next->_value = (char *) malloc((strlen(node->_value) * sizeof(char)) + 1);
            if(cursor->_next->_value == NULL)return false;
            strcpy(cursor->_next->_value, node->_value);
            delete node;

            return true;
        }
        cursor = cursor->_next;
    }
    cursor->_next = node;

    return true;
}

boolean Dictionary::deleteParameter(const char *parameter)
{
    if(_head->_next == NULL) return false;

    Dictionary *cursor = _head, *toDelete(NULL);

    while(!isListEmpty(cursor->_next))
    {
        if(strcmp(cursor->_next->_parameter, parameter) == 0)
        {
            toDelete = cursor->_next;
            cursor->_next = cursor->_next->_next;
            delete toDelete;
            return true;
        }
        cursor = cursor->_next;
    }
    return false;
}

void Dictionary::dispose()
{
    if(isListEmpty(_head->_next))return;

    Dictionary *cursor = _head->_next, *toDelete(NULL);

    while(!isListEmpty(cursor))
    {
        toDelete = cursor;
        cursor = cursor->_next;

        delete toDelete;
    }
    _head = this;
    _next = NULL;
}

void Dictionary::clear() {this->dispose();}

