#ifndef DICTIONARY_H
#define DICTIONARY_H

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <Arduino.h>

class Dictionary
{
public:
    ~Dictionary();

    boolean addParameter(const char *parameter, const char *value);
    boolean addParameter(const char *parameter, const int value)
    {
        char buffer[12] = "";
        sprintf(buffer,"%d",value);
        return addParameter(parameter, buffer);
    };
    boolean addParameter(const char *parameter, const float value)
    {
        return addParameter(parameter, (double) value);
    };
    boolean addParameter(const char *parameter, const double value)
    {
        char buffer[50] = "";
        sprintf(buffer,"%f",value);
        return addParameter(parameter, buffer);
    };
    boolean deleteParameter(const char *parameter);
    Dictionary get(const char *parameter);
    Dictionary operator()(const char *parameter);
    Dictionary get(const unsigned int index);
    Dictionary operator()(const unsigned int index);
    unsigned int count();
    void clear();
    void dispose();

    long longValue() const {return _value == NULL ? 0 : strtol(_value, NULL, 10);}
    int intValue() const {return (int) (_value == NULL ? 0 : strtol(_value, NULL, 10));}
    unsigned long uintValue() const {return _value == NULL ? 0 : strtoul(_value, NULL, 10);}
    const char *stringValue() const {return _value == NULL ? "" : _value;}
    double doubleValue() const {return _value == NULL ? 0 : strtod(_value, NULL);}
    float floatValue() const {return _value == NULL ? 0 : strtof(_value, NULL);}
    boolean booleanValue() const
    {
        if(_value == NULL)
            return false;
        return strcmp(_value,"true") == 0 || strcmp(_value,"TRUE") == 0 ? true : false;
    }
    const char *getParameter() const{return _parameter == NULL ? "" : _parameter;}
protected:
    Dictionary();
    Dictionary(const char *parameter, const char *value);
    Dictionary(Dictionary const& dictionaryToCopy); //Copy constructor

    boolean addNewNodeAtTheEnd(Dictionary *node);
    boolean isListEmpty(Dictionary *node) {return node == NULL;}

    char *_parameter;
    char *_value;
    Dictionary *_next;
    Dictionary *_head;
private:
};

#endif //DICTIONARY_H
